# coding: utf-8

from argparse import ArgumentParser
from names import MIDDLE_NAMES
from names import SYSTEMS
from objects import Organization
from objects import Person
from random import randint
from suds.client import Client
from suds.xsd.doctor import Import
from suds.xsd.doctor import ImportDoctor


def param_parser():
    u"""
    Добавляет параметры для формирования данных.
    :return: Возвращает подготовленные к использованию значения.
    :rtype: dict
    """
    arg_parser = ArgumentParser(
        description=u'Генератор данных для Контингент'
    )
    arg_parser.add_argument(
        '--persons',
        '-p',
        type=int,
        default=1,
        help=u'Количество генерируемых блоков Person.'
             u'По умолчанию будет создан 1 Peron'
    )
    arg_parser.add_argument(
        '--organizations',
        '-c',
        type=int,
        default=0,
        help=u'Количество дополнительно генерируемых блоков Organization. '
             u'По умолчанию будет создана только 1 Organization.'
    )
    arg_parser.add_argument(
        '--region',
        '-reg',
        type=str,
        help=u'Регион для создаваемых учреждений.'
             u' Например: Мурманская область.'
    )
    arg_parser.add_argument(
        '--city',
        type=str,
        help=u'Город для создаваемых учреждений.'
             u' Например: г. Мурманск'
    )
    arg_parser.add_argument(
        '--oktmo',
        type=int,
        help=u'Код ОКТМО для создаваемых учреждений.'
    )
    arg_parser.add_argument(
        '--system',
        type=str,
        help=u'ИС-источник формируемых данных.'
             u' Вомзожны варианты: "ЭДС", "ЭШ", "ЭДО", "ЭПО", "ЭК". '
             u' Возможно указать список систем: "ЭДС, ЭШ, ЭДО" и т.п. '
             u' Для формирования данных по всем системам используется "ALL"'
    )
    arg_parser.add_argument(
        '--send',
        type=str,
        default='True',
        help=u'Производить ли отправку данных после формирования.'
             u' По умолчанию = True.'
    )
    arg_parser.add_argument(
        '--type',
        type=int,
        default=0,
        help=u'Способ формирования ДУЛ для персон.'
             u' Возможны варианты: 0, 1, 2 или 3'
    )
    opt, _ = arg_parser.parse_known_args()
    opt = vars(opt)
    is_fall, message = check_params(opt)
    if is_fall:
        raise KeyError(message)
    else:
        if opt['organizations'] in (0, 1):
            opt['organizations'] = 0
        else:
            opt['organizations'] -= 1

        opt['region'] = opt['region'].decode('utf-8')
        opt['city'] = opt['city'].decode('utf-8')
        opt['system'] = opt['system'].decode('utf-8').upper()
    return opt


def check_system_parameter(system_name, system_list):
    system_name = system_name.split(',')
    if len(system_name) == 1:
        if system_name[0] not in system_list and system_name[0] != 'ALL':
            return False
    else:
        if 'ALL' in system_name:
            return False
        elif not all(map(
                lambda sys_name: sys_name.strip() in system_list,
                system_name
        )):
            return False
    return True


def check_params(params):
    u"""
    Проверяет корректность данных.
    :param params: Параметры запуска формирования.
    :return: Триггер наличия ошибки и сообщение.
    :rtype: tuple
    """
    is_fall = False
    err_message = ''
    if not params['region']:
        is_fall = True
        err_message = 'Need --region'
    elif not params['city']:
        is_fall = True
        err_message = 'Need --city'
    elif not params['oktmo']:
        is_fall = True
        err_message = 'Need --oktmo'
    elif not params['system']:
        is_fall = True
        err_message = 'Need --system'
    elif not check_system_parameter(
            params['system'].decode('utf-8').upper(),
            SYSTEMS.keys()
    ):
        is_fall = True
        err_message = 'Need correct system name. Use data_generator --help'
    elif params['type'] not in (0, 1, 2, 3):
        is_fall = True
        err_message = 'Need correct value for --type. Use data_generator --help'

    return (is_fall, err_message)


def get_middle_name():
    u"""
    :return: Возвращает отчество
    :rtype: unicode
    """
    count = len(MIDDLE_NAMES)-1
    return MIDDLE_NAMES[randint(0, count)]


def create_organization(region, city, oktmo, system, faker):
    u"""
    :param region: Регион для организации
    :param city: Город для организации
    :param oktmo: Код ОКТМО для органиации
    :param system: ИС-источник
    :param faker: Объект фабрики генератора faker
    :return: Возвращает объект организации
    """
    return Organization(
        region=region, city=city,
        oktmo=oktmo, system=system,
        factory=faker
    )


def create_person(fake, organization, system, is_all,
                  need_health, is_the_last_system, dul_gen_type):
    u"""
    :param fake: Объект фабрики генератора faker
    :param organization: Объект организации
    :param system: ИС-источник
    :param is_all: Данные по всем системам
    :param need_health: Генерация данных по здоровью
    :param is_the_last_system: Данные для последний системы
    :param dul_gen_type: Тип формирования ДУЛ персоны
    :return: Возвращает объект персоны
    """
    firstname = fake.first_name_male()
    lastname = fake.last_name_male()
    middlename = get_middle_name()
    del_firstname = fake.first_name_male()
    del_lastname = lastname
    del_middlename = get_middle_name()

    return Person(
        firstname, middlename, lastname,
        del_firstname, del_middlename, del_lastname,
        organization, system, is_all, need_health,
        is_the_last_system, dul_gen_type
    )


def get_soap_client(url):
    u"""
    :param url: Url-адрес для soap-клиента
    :return: Возвращает сконфигурированный soap-клиент
    """
    schema_url = 'http://bars-open.ru/inf'
    schema_import = Import(schema_url)
    schema_doctor = ImportDoctor(schema_import)
    return Client(url=url, doctor=schema_doctor)


def check_systems_order(sys_list):
    sys_list = [sys_name.upper() for sys_name in sys_list]
    dou = sys_list.index(u'ЭДС') if u'ЭДС' in sys_list else -1
    school = sys_list.index(u'ЭШ') if u'ЭШ' in sys_list else -1
    ssuz = sys_list.index(u'ЭК') if u'ЭК' in sys_list else -1
    if len(sys_list) > 1:
        if ssuz > 0:
            if ssuz < school or ssuz < dou:
                return False
        elif school > 0:
            if school < dou:
                return False
    return True
